import { loadModules } from 'esri-loader'
import MAP_OPTIONS from './config/baseConfig'
import { Subject } from 'rxjs'
import SgLayerFactory from './manage/layer/SgLayerFactory'

export default class SgMap {
  /**
   * kelsey yau
   * @param {*} id 类实例的的id
   * @param {*} mapConfig 地图的基本配置
   * @param {*} container 地图的容器
   * @param {*} styleConfg 样式的配置
   */
  constructor (id, mapConfig, container, styleConfig) {
    this.id = id
    this.mapConfig = mapConfig
    this.container = container
    this.styleConfg = styleConfig || null
    this.maplife = new Subject()
    this.drawLayerId = 'drawLayer'
    this.highlightLayerId = 'highlightLayer'
    loadModules([
      'esri/Map',
      'esri/views/MapView',
      'esri/layers/GraphicsLayer',
      'esri/layers/GeoJSONLayer',
      'esri/Graphic',
      'esri/geometry/SpatialReference',
      'esri/geometry/Extent',
      'esri/tasks/GeometryService',
      'esri/geometry/Point',
      'esri/geometry/Polyline',
      'esri/geometry/Polygon',
      'esri/geometry/Circle',
      'esri/core/watchUtils'
    ], MAP_OPTIONS).then(([Map, MapView, GraphicsLayer, GeoJSONLayer,
      Graphic, SpatialReference, Extent, GeometryService, Point, Polyline,
      Polygon, Circle, watchUtils
    ]) => {
      // 播发模块已经加载
      this.maplife.next('before-create')

      // 地图类实例初始化
      this.map = new Map({})

      // 地图范围记录器
      this.extentRecord = {
        prevState: false,
        currentExtent: null,
        preExtent: null,
        historyExtent: [],
        historyExtentIdx: 0,
        nextState: false
      }

      // 添加初始化图层
      if (this.mapConfig.webgisBasemap) {
        this.mapConfig.webgisBasemap.forEach((value) => {
          SgLayerFactory.addConfigLayer(value, this)
        })
      }

      // 坐标系统对象
      this.sr = new SpatialReference(this.mapConfig.webgisSr)

      // 视图范围对象
      this.extent = new Extent(this.mapConfig.webgisExtent)
      this.extent.spatialReference = this.sr

      // 地图视图类实例初始化
      this.mapView = new MapView({
        container: container,
        map: this.map,
        extent: this.extent,
        center: this.mapConfig.webgisInitCenter,
        zoom: this.mapConfig.webgisInitLevel
      })

      // 移除默认的地图ui组件
      this.mapView.ui.remove(['attribution', 'zoom'])

      // 图层类实例初始化
      this.graphicsLayer = new GraphicsLayer()
      this.geojsonLayer = new GeoJSONLayer()

      // 操作类图层实例
      this.highlightLayer = new GraphicsLayer({
        id: this.highlightLayerId,
        spatialReference: this.mapView.spatialReference
      })
      this.drawLayer = new GraphicsLayer({
        id: this.drawLayerId,
        spatialReference: this.mapView.spatialReference
      })

      // 地图要素类实例初始化
      this.graphic = new Graphic()

      // 地图几何服务初始化
      if (this.mapConfig.webgisGeo) {
        this.geometryService = new GeometryService({
          url: this.mapConfig.webgisGeo
        })
      }

      // 地图几何对象初始化
      this.Point = Point
      this.Polyline = Polyline
      this.Polygon = Polygon
      this.Circle = Circle

      // 监听工具
      this.watchUtils = watchUtils

      this.mapView.when(() => {
        // 播发地图初始化
        this.maplife.next('created')
        this._initMapExtentEvent()
      }, () => {
        console.log('=============================》地图初始化失败')
      })
    }, error => {
      console.log('=============================》模块获取失败', error)
    })
  }

  // 地图的生命周期方法

  /**
   * 地图实例创建之前，异步库模块已经加载完成
   * @param {*} callback 回调函数
   */
  beforeMapCreate (callback) {
    this.maplife.subscribe((value) => {
      if (value === 'before-create') {
        callback()
      }
    })
  }

  /**
   * 地图初始化完成
   * @param {*} callback 地图实例创建完成的回调函数
   */
  mapCreated (callback) {
    this.maplife.subscribe((value) => {
      if (value === 'created') {
        callback()
      }
    })
  }

  _initMapExtentEvent () {
    this.watchUtils.whenOnce(this.mapView, 'extent', () => {
      this.watchUtils.when(this.mapView, 'stationary', (evt) => {
        if (evt) {
          this._mapExtentRecordListener(this.mapView.extent)
        }
      })
    })
  }

  _mapExtentRecordListener (evt) {
    if (this.extentRecord.prevState || this.extentRecord.nextState) {
      this.extentRecord.currentExtent = evt
      this.extentRecord.prevState = this.extentRecord.nextState = false
    } else {
      this.extentRecord.preExtent = this.extentRecord.currentExtent
      this.extentRecord.currentExtent = evt
      this.extentRecord.historyExtent.push({
        preExtent: this.extentRecord.preExtent,
        currentExtent: this.extentRecord.currentExtent
      })
      if (this.extentRecord.historyExtent.length > 10) {
        this.extentRecord.historyExtent.shift()
      }
      this.extentRecord.historyExtentIdx = this.extentRecord.historyExtent.length - 1
      this.extentRecord.prevState = this.extentRecord.nextState = false
    }
  }
}
