import SgManager from '../SgManager'
import SgLayerFactory from './SgLayerFactory'

export default class SgLayerManager extends SgManager {
  /**
   * kelsey yau
   * 添加图层对象到地图中
   * @param {*} layerObject 图层对象，支持所有类型图层，
   * 见https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html
   * @param {*} index 图层的顺序
   */
  addLayer (layerObject, index) {
    this.sgMap.map.add(layerObject, index)
  }
  /**
   * kelsey yau
   * 添加配置图层，传入图层配置，单个图层添加或多个图层添加
   * @param {*} layerConfig object || array 图层配置数据
   */
  addConfigLayer (layerConfig) {
    if (layerConfig instanceof Array) {
      layerConfig.forEach((value) => {
        SgLayerFactory.addConfigLayer(value, this.sgMap)
      })
    } else {
      SgLayerFactory.addConfigLayer(layerConfig, this.sgMap)
    }
  }

  /**
   * kelsey yau
   * 矢量图层添加要素
   * @param {*} layerId string 图层Id
   * @param {*} graphics object || array<object> 要添加的要素,可以添加单个要素，多个要素
   */
  addFeatureToVectorLayer (layerId, graphics) {
    const layer = this.getLayerById(layerId)
    const graphicsCollection = layer.graphics
    if (graphics instanceof Array) {
      graphicsCollection.addMany(graphics)
    } else {
      graphicsCollection.add(graphics)
    }
  }

  /**
   * 判断一个图层是否为可查询图层
   * @param {*} layer 图层对象
   */
  judgeArcgisQueryLayer (layer) {
    const type = layer.type
    return type === 'map-image' || type === 'feature' || type === 'tile'
  }

  /**
   * kelsey yau
   * 判断地图中是否有对应图层id的图层
   * @param {*} layerId string 图层id
   */
  hasLayerById (layerId) {
    return !! this.getLayerById(layerId)
  }

  /**
   * kelsey yau
   * 矢量图层移除制定属性id值的要素
   * @param {*} layerId string 图层Id
   * @param {*} attrId string 要素的属性Id值
   */
  removeFeatureInVectroLayer (layerId, attrId) {
    const layer = this.getLayerById(layerId)
    const graphicsCollection = layer.graphics
    const graphic = graphicsCollection.find((graphic) => {
      return graphic.attributes.id === attrId
    })
    graphicsCollection.remove(graphic)
  }

  /**
   * 移除矢量图层的所有要素
   * @param {*} layerId string 图层Id
   */
  removeAllFeatureInVectorLayer (layerId) {
    const layer = this.getLayerById(layerId)
    const graphicsCollection = layer.graphics
    graphicsCollection.removeAll()
  }

  /**
   * kelsey yau
   * 创建一个矢量图层
   * @param {*} id string 图层Id
   * @param {*} graphics Collection<Graphic> 图层的要素
   */
  createVectorLayer (id, graphics) {
    const newVectorLayer = this.sgMap.graphicsLayer.clone()
    newVectorLayer.id = id
    if (graphics) { newVectorLayer.graphics = graphics }
  }

  /**
   * kelsey yau
   * 根据图层Id获取图层对象
   * @param {*} layerId string 图层的Id
   */
  getLayerById (layerId) {
    const layer = this.sgMap.map.findLayerById(layerId)
    return layer
  }

  /**
   * 易进坤
   * 根据图层Id和子图层Id获取子图层
   * @param layerId 图层Id (String)
   * @param subLayerId 子图层Id (Number)
   * @returns {*} 子图层 (esri/layers/support/Sublayer)
   */
  getSubLayerById(layerId,subLayerId){
    const layer = this.sgMap.map.findLayerById(layerId)
    return layer.allSublayers.find(function(sublayer){
      return sublayer.id === subLayerId
    });
  }

  /**
   * 易进坤
   * 根据图层Id和子图层Id设置子图层是否显示
   * @param layerId 图层Id (String)
   * @param subLayerId 子图层Id (Number)
   * @param visible 是否显示（Boolean）
   */
  setSubLayerVisibleById(layerId,subLayerId,visible){
    this.getSubLayerById(layerId,subLayerId).visible = visible
  }

  /**
   * kelsey yau
   * 获取地图的所有图层,返回arcgis collection对象，
   * 见https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html
   */
  getAllLayers () {
    return this.sgMap.map.allLayers
  }

  /**
   * 获取所有的可见图层
   */
  getAllVisibleLayer () {
    let visibleLayerCollection = []
    const layerCollection = this.getAllLayers()
    layerCollection.forEach((layer) => {
      if (layer && layer.visible) {
        visibleLayerCollection.push(layer)
      }
    })
    return visibleLayerCollection
  }
  /**
   * kelsey yau
   * 根据地图的图层Id获取地图的比例尺范围
   * @param {*} layerId string 图层id
   */
  getLayerScaleById (layerId) {
    const layer = this.getLayerById(layerId)
    return [layer.minScale, layer.maxScale]
  }

  /**
   * kelsey yau
   * 根据图层Id移除图层
   * @param {*} layerId string 图层Id
   */
  removeLayerById (layerId) {
    const layer = this.getLayerById(layerId)
    this.sgMap.map.remove(layer)
  }

  /**
   * kelsey yau
   * 移除所有图层
   */
  removeAllLayers () {
    this.sgMap.map.removeAll()
  }

  /**
   * kelsey yau
   * 控制单个或多个图层的显示隐藏
   * @param {*} layerIds string || array<string> 图层id或者图层id数组
   * @param {*} visibility boolean 图层显示隐藏，true显示，false隐藏
   */
  setLayerVisibility (layerIds, visibility) {
    if (layerIds instanceof Array) {
      layerIds.forEach((layerId) => {
        const layer = this.getLayerById(layerId)
        if (layer) {
          layer.visible = visibility
        }
      })
    } else {
      const layer = this.getLayerById(layerIds)
      if (layer) {
        layer.visible = visibility
      }
    }
  }

  /**
   * kelsey yau
   * 根据图层的id设置对应图层的透明度
   * @param {*} layerId string 图层id
   * @param {*} opacity number 透明度，范围0~1
   */
  setLayerOpacityById (layerId, opacity) {
    const layer = this.getLayerById(layerId)
    layer.opacity = opacity
  }

  /**
   * kelsey yau
   * 根据图层的id设置图层比例尺范围
   * @param {*} layerId string 图层的id
   * @param {*} scaleRange array<number> 比例尺范围,最小比例尺和最大比例尺，[min, max]
   */
  setLayerScaleById (layerId, scaleRange) {
    const layer = this.getLayerById(layerId)
    layer.minScale = scaleRange[0]
    layer.maxScale = scaleRange[1]
  }
}
