import SgManager from '../SgManager'
import SgEvent from '../../event/Event'
import { loadModules } from 'esri-loader'
import MAP_OPTIONS from '../../config/baseConfig'

export default class SgViewManager extends SgManager {
  constructor (sgMap) {
    super(sgMap)
    this.fullscreenViewModel = null
    this._ZOOMIN_RATIO = 0.5 // 放大比率
    this._ZOOMOUT_RATIO = 2 // 缩小比率
    this._DEFAULT_ROTATION = 0 // 默认地图旋转角度
    this.event = new SgEvent(this.sgMap.mapView) // 创建视图事件类
  }

  /**
   * 视图管理单例模式
   * @param {*} sgMap SgMap对象实例
   */
  static getInstance (sgMap) {
    if (!this.instance) {
      this.instance = new SgViewManager(sgMap)
    }
    return this.instance
  }

  /**
   * 改变地图视图容器的鼠标样式
   * @param {*} type string 鼠标样式类型
   */
  changeMapContainerCursor (type) {
    const containerEle = this.getViewContainer()
    containerEle.style.cursor = type
  }

  /**
   * 获取地图视图容器
   */
  getViewContainer () {
    return this.sgMap.mapView.container
  }

  // 获取地图中心坐标
  getCenter () {
    const x = this.sgMap.mapView.center.x
    const y = this.sgMap.mapView.center.y
    return [x, y]
  }

  /**
   * 获取几何点对应的屏幕坐标对象 {x, y}
   * @param {*} point object 几何点对象
   */
  getScreenCoor (point) {
    return this.sgMap.mapView.toScreen(point)
  }
  // 获取当前视图范围
  getExtent () {
    const extent = this.sgMap.mapView.extent
    return extent
  }

  /**
   * kelsey yau
   * 获取当前地图的比例尺
   */
  getMapScale () {
    return this.sgMap.mapView.scale
  }

  /**
   * 获取地图当前显示的等级, 用于等级的图层，如瓦片图
   */
  getMapZoom () {
    return this.sgMap.mapView.zoom
  }

  /**
   * 获取地图的容器的宽高尺寸
   */
  getViewContainerSize () {
    return {
      width: this.sgMap.mapView.width,
      height: this.sgMap.mapView.height
    }
  }

  /**
   * 根据屏幕点获取要素，返回一个promise对象
   * @param {*} point 屏幕点坐标
   */
  getFeaturesAtScreenPoint (point) {
    return this.sgMap.mapView.hitTest(point)
  }

  /**
   * kelsey yau
   * 跳转至设置中心位置
   * @param {*} center 地图的中心位置，如 [571994.61223, 2497703.079702]
   */
  setCenter (center) {
    // this.sgMap.mapView.center = center
    this.sgMap.mapView.goTo(center)
  }

  // 要素居中
  setCenterFeature (feature) {
    this.sgMap.mapView.goTo(feature)
  }

  /**
   * kelsey yau
   * 设置地图比例尺
   * @param {*} scale number 比例尺
   */
  setMapScale (scale) {
    this.sgMap.mapView.scale = scale
  }

  /**
   * kelsey yay
   * 地图放大
   */
  setMapZoomIn () {
    let scale = this.getMapScale() * this._ZOOMIN_RATIO
    this.setMapScale(scale)
  }

  /**
   * 地图缩小
   */
  setMapZoomOut () {
    let scale = this.getMapScale() * this._ZOOMOUT_RATIO
    this.setMapScale(scale)
  }

  /**
   * 设置为初始全图
   */
  setFullMap () {
    this.sgMap.mapView.extent = this.sgMap.extent
    this.sgMap.mapView.rotation = this._DEFAULT_ROTATION
  }

  /**
   * 设置地图的等级，用于arcgis瓦片图层、wmts、矢量瓦片、wms、互联网瓦片，arcgis动态、feature
   * 图层设置无效
   * @param {*} number zoom 地图的等级
   */
  setMapZoom (zoom) {
    this.sgMap.mapView.zoom = zoom
  }

  /**
   * 设置上一视图
   */
  setPrevView () {
    let currentIdx = this.sgMap.extentRecord.historyExtentIdx
    if (this.sgMap.extentRecord.historyExtent[currentIdx].preExtent) {
      if (currentIdx > 0) {
        this.sgMap.extentRecord.prevState = true
        this.sgMap.mapView.goTo(this.sgMap.extentRecord.historyExtent[currentIdx].preExtent)
        this.sgMap.extentRecord.historyExtentIdx--
      }
    }
  }

  /**
   * 设置下一视图
   */
  setNextView () {
    let currentIdx = this.sgMap.extentRecord.historyExtentIdx
    let extentLength = this.sgMap.extentRecord.historyExtent.length
    if (currentIdx < extentLength - 1) {
      this.sgMap.extentRecord.historyExtentIdx++
      if (this.sgMap.extentRecord.historyExtent[this.sgMap.extentRecord.historyExtentIdx]) {
        this.sgMap.extentRecord.nextState = true
        this.sgMap.mapView.goTo(this.sgMap.extentRecord.historyExtent[this.sgMap.extentRecord.historyExtentIdx].currentExtent)
      }
    }
  }

  /**
   * 设置全图
   */
  setFullScreen () {
    if (this.fullscreenViewModel === null) {
      loadModules(['esri/widgets/Fullscreen/FullscreenViewModel'], MAP_OPTIONS).then(([FullscreenViewModel]) => {
        this.fullscreenViewModel = new FullscreenViewModel({
          element: this.getViewContainer(),
          view: this.sgMap.mapView
        })
        this.fullscreenViewModel.enter()
      })
    } else {
      this.fullscreenViewModel.enter()
    }
  }

  /**
   * kelsey yau
   * 添加视图事件绑定
   * @param {*} evenObj object 事件对象
   */
  on (evenObj) {
    this.event.on(evenObj)
  }

  /**
   * kelsey yau
   * 事件解绑
   * @param {*} evenObj object 事件对象
   */
  un (evenObj) {
    this.event.un(evenObj)
  }

  /**
   * 销毁所有绑定事件
   */
  destroy () {
    this.event.destroy()
  }
}
