import SgOperation from './SgOperation'
import { loadModules } from 'esri-loader'
import { createGraphic, createGeometry } from '../utils/SgMapUtils'
import { Subject } from 'rxjs'
import MAP_OPTIONS from '../config/baseConfig'
import { styleConfig } from '../config/styleConfig'

export default class SgBufferOperation extends SgOperation {
  constructor (sgMap) {
    super(sgMap)
    this._bufferSubject = new Subject()
    this._bufferSubscrip = null
    loadModules([
      'esri/geometry/geometryEngineAsync',
      'esri/geometry/Polyline'
    ], MAP_OPTIONS).then(([geometryEngineAsync, Polyline]) => {
      this.geometryEngineAsync = geometryEngineAsync
      this.Polyline = Polyline
      console.log(this.geometryEngineAsync)
    })
  }

  static getInstance (sgMap) {
    if (!this.instance) {
      this.instance = new SgBufferOperation(sgMap)
    }
    return this.instance
  }

  /**
   * 创建客户端缓冲区
   * @param {*} geometry 要进行缓冲区的几何对象，可以是一个几何对象或几何对象数组
   * @param {*} distance number 缓冲区的距离，单位是米
   */
  createBuffer (geometry, distance) {
    this.clearBufferLayer()
    this._initAnalyseLayer()
    let bufferGeometry = this._initGeometry(geometry)
    if (this.map.sr.wkid === '4326' || this.map.sr.wkid === '3857') {
      this.geometryEngineAsync.geodesicBuffer(bufferGeometry, distance, 'meters').then((response) => {
        this._bufferSubject.next(response)
        this._addBufferToLayer(response)
      })
    } else {
      this.geometryEngineAsync.buffer(bufferGeometry, distance, 'meters').then((response) => {
        this._bufferSubject.next(response)
        this._addBufferToLayer(response)
      })
    }
  }

  /**
   * 通过arcgisServer geometry服务进行缓冲区的创建
   * @param {*} geometry 要进行缓冲区的几何对象，可以是一个几何对象或几何对象数组
   * @param {*} distance number 缓冲区的距离，单位是米
   */
  createGeometryServiceBuffer (geometry, distance) {
    loadModules(['esri/tasks/support/BufferParameters'], MAP_OPTIONS).then(([BufferParameters]) => {
      this._initAnalyseLayer()
      this.clearBufferLayer()
      let bufferGeometry = this._initGeometry(geometry)
      let inputGeometry
      bufferGeometry instanceof Array ? inputGeometry = bufferGeometry : inputGeometry = [bufferGeometry]
      let params = new BufferParameters({
        distance: [distance],
        units: 'meters',
        geometries: inputGeometry
      })
      this.map.geometryService.buffer(params).then((result) => {
        this._bufferSubject.next(result)
        this._addBufferToLayer(result)
      })
    })
  }

  /**
   * 监听缓冲区绘制
   * @param {*} callback 缓冲区绘制回调函数
   */
  onBuffer (callback) {
    this._bufferSubscrip = this._bufferSubject.subscribe((value) => {
      callback(value)
    })
  }

  /**
   * 取消监听缓冲区绘制
   */
  unBuffer () {
    this._bufferSubscrip.unsubscribe()
  }

  /**
   * 清除缓冲区绘制图层
   */
  clearBufferLayer () {
    this.map.drawLayer.removeAll()
  }

  /**
   * 初始空间分析图层，私有方法
   */
  _initAnalyseLayer () {
    if (!this.layerManager.hasLayerById(this.map.drawLayerId)) {
      this.layerManager.addLayer(this.map.drawLayer)
    }
  }

  /**
   * 添加创建的缓冲区到图层中，私有方法
   * @param {*} bufferPolygon 缓冲区多边形
   */
  _addBufferToLayer (bufferPolygon) {
    if (bufferPolygon instanceof Array) {
      let graphics = []
      bufferPolygon.forEach((polygon) => {
        let graphic = createGraphic(this.map, polygon, styleConfig.BUFFER_POLYGON)
        graphics.push(graphic)
      })
      this.map.drawLayer.addMany(graphics)
    } else {
      let graphic = createGraphic(this.map, bufferPolygon, styleConfig.BUFFER_POLYGON)
      this.map.drawLayer.add(graphic)
    }
  }

  /**
   * 初始化几何要素，判断输入几何要素是否为arcgis geometry对象以及是否含坐标系，
   * 并且将要进行缓冲区的要素绘制到图层上
   * @param {*} geometry 输入的几何要素或几何要素数组
   */
  _initGeometry (geometry) {
    let tempGeometry, tempGraphic
    if (geometry instanceof Array) {
      tempGeometry = []
      tempGraphic = []
      geometry.forEach((geom) => {
        if (!geom.spatialReference) {
          geom.spatialReference = this.map.sr
        }
        let bufferGeom = createGeometry(this.map, geom)
        let bufferGraphic = createGraphic(this.map, bufferGeom)
        tempGeometry.push(bufferGeom)
        tempGraphic.push(bufferGraphic)
      })
      this.map.drawLayer.addMany(tempGraphic)
    } else {
      if (!geometry.spatialReference) {
        geometry.spatialReference = this.map.sr
      }
      tempGeometry = createGeometry(this.map, geometry)
      tempGraphic = createGraphic(this.map, tempGeometry)
      this.map.drawLayer.add(tempGraphic)
    }
    return tempGeometry
  }
}
