import { loadModules } from 'esri-loader'
import SgOperation from './SgOperation'
import SgEvent from '../event/Event'
import { Subject } from 'rxjs'
import MAP_OPTIONS from '../config/baseConfig'
import { createPointGraphic, createLineGraphic, createPolygonGraphic, createCircleGraphic, createRetangleGraphic } from '../utils/SgMapUtils'
import SgViewManager from '../manage/view/SgViewManager'
import { styleConfig } from '../config/styleConfig'
import SgMapOverlayer from '../overlayer/SgMapOverlayer'

export default class SgDrawOperation extends SgOperation {
  constructor (sgMap) {
    super(sgMap)
    this.viewManager = SgViewManager.getInstance(this.map)
    this.drawLayer = this.map.drawLayer
    this.drawSubject = new Subject()
    this.drawSubscription = null
    this.sketchVM = null
    this.currentDrawType = null
    this.sgOverlay = SgMapOverlayer.getInstance(this.map)
    loadModules([
      'esri/widgets/Sketch/SketchViewModel',
      'esri/views/draw/Draw'
    ], MAP_OPTIONS).then(([SketchViewModel, Draw]) => {
      this.draw = new Draw({
        view: this.map.mapView
      })
      this.SketchVM = SketchViewModel
    })
    this.drawListener = (event) => {
      if (event.button === 2) {
        this.draw.reset()
        this.map.mapView.graphics.removeAll()
      }
    }
  }

  /**
   * 创建单次绘制
   * @param {*} type 绘制的类型，包括'point'，'polyline'，'polygon'，'rectangle'，'circle'
   */
  createDraw (type) {
    this.sketchVM = new this.SketchVM({
      view: this.map.mapView,
      layer: this.drawLayer,
      pointSymbol: styleConfig.DRAW_POINT,
      polylineSymbol: styleConfig.DRAW_LINE,
      polygonSymbol: styleConfig.DRAW_POLYGON
    })
    let drawLayer = this.layerManager.getLayerById(this.map.drawLayerId)
    if (!drawLayer) { this.layerManager.addLayer(this.drawLayer) }
    if (this.event) {
      this.event.destroy()
      this.event = null
    }
    this.event = new SgEvent(this.sketchVM)
    this.event.on({ 'create': (event) => {
      switch (event.state) {
        case 'start':
          this.drawSubject.next({ type: 'start', result: event })
          break
        case 'active':
          this.drawSubject.next({ type: 'active', result: event })
          break
        case 'complete':
          this.drawSubject.next({ type: 'complete', result: event })
          break
        default:
          return null
      }
    } })
    this.sketchVM.create(type)
  }

  /**
   * 创建常规可执行多次绘制
   * @param {*} type 绘制的类型：'point','polyline','polygon','circle','rectangle'
   */
  createCustomDraw (type) {
    this.currentDrawType = type
    let drawLayer = this.layerManager.getLayerById(this.map.drawLayerId)
    if (!drawLayer) { this.layerManager.addLayer(this.drawLayer) }
    this.draw.reset()
    let drawAuction = this.draw.create(type, { mode: 'click' })
    if (type === 'point') {
      drawAuction.on('cursor-update', this._drawPoint.bind(this))
      drawAuction.on('draw-complete', this._addToDrawLayer.bind(this))
    } else if (type === 'polyline') {
      drawAuction.on('cursor-update', this._drawLine.bind(this))
      drawAuction.on('vertex-add', this._drawLine.bind(this))
      drawAuction.on('vertex-remove', this._drawLine.bind(this))
      drawAuction.on('draw-complete', this._addToDrawLayer.bind(this))
    } else if (type === 'polygon') {
      drawAuction.on('cursor-update', this._drawPolygon.bind(this))
      drawAuction.on('vertex-add', this._drawPolygon.bind(this))
      drawAuction.on('vertex-remove', this._drawPolygon.bind(this))
      drawAuction.on('draw-complete', this._addToDrawLayer.bind(this))
    } else {
      drawAuction.on('cursor-update', this._drawSegment.bind(this))
      drawAuction.on('vertex-add', this._drawSegment.bind(this))
      drawAuction.on('draw-complete', this._addToDrawLayer.bind(this))
    }
  }

  /**
   * 监听绘制
   * @param {*} eventObj 事件对象
   */
  on (eventObj) {
    this.drawSubscription = this.drawSubject.subscribe((value) => {
      for (let type in eventObj) {
        if (type === value.type) {
          eventObj[type](value.result)
        }
      }
    })
  }

  /**
   * 撤销绘制
   */
  cancelDraw () {
    this.draw.reset()
    this.map.mapView.graphics.removeAll()
  }

  /**
   * 清除绘制
   */
  clearDraw () {
    this.cancelDraw()
    this.drawLayer.removeAll()
    this.map.mapView.popup.close()
  }

  /**
   * 将绘制结果加到图层中
   * @param {*} event 回调参数
   */
  _addToDrawLayer (event) {
    let vertices
    let type = this.currentDrawType
    let drawLayer = this.layerManager.getLayerById(this.map.drawLayerId)
    if (!drawLayer) { this.layerManager.addLayer(this.drawLayer) }
    this.map.mapView.graphics.removeAll()
    let graphic
    switch (type) {
      case 'point':
        if (event.native.button === 2) {
          this.draw.reset()
          // this.sgOverlay.clearInfoWindowById('drawpoint')
        } else {
          vertices = event.coordinates
          graphic = this._createPoint({
            x: vertices[0],
            y: vertices[1]
          })
          drawLayer.add(graphic)
        }
        break
      case 'polyline':
        vertices = event.vertices
        graphic = this._createLine(vertices)
        drawLayer.add(graphic)
        break
      case 'polygon':
        vertices = event.vertices
        graphic = this._createPolygon(vertices)
        drawLayer.add(graphic)
        break
      case 'circle':
        vertices = event.vertices
        graphic = this._createCircle(vertices)
        drawLayer.add(graphic)
        break
      case 'rectangle':
        vertices = event.vertices
        graphic = this._createRectangle(vertices)
        drawLayer.add(graphic)
        break
      default:
        break
    }
    this.drawSubject.next({ type: 'complete', result: graphic })
    if (event.native.button !== 2) {
      this.createCustomDraw(type)
    }
  }

  /**
   * 绘制点
   * @param {*} event 回调参数
   */
  _drawPoint (event) {
    // this.sgOverlay.addInfoWindow(
    //   { x: 0, y: 0, spatialReference: this.map.sr, type: 'point' },
    //   { id: 'drawpoint', content: '左击绘制点<br/>右击结束' }
    // )
    let vertices = event.coordinates
    this.map.mapView.graphics.removeAll()
    let point = {
      x: vertices[0],
      y: vertices[1]
    }
    let graphic = this._createPoint(point)
    this.map.mapView.graphics.add(graphic)
    // this.sgOverlay.updateInfoWindow('drawpoint', graphic.geometry)
  }

  /**
   * 绘制线
   * @param {*} event 回调参数
   */
  _drawLine (event) {
    this._drawStartClick()
    let vertices = event.vertices
    this.map.mapView.graphics.removeAll()
    let startPoint = this._createPoint({ x: vertices[0][0], y: vertices[0][1] })
    let line = this._createLine(vertices)
    this.map.mapView.graphics.addMany([line, startPoint])
    for (let i = 1; i < vertices.length; i++) {
      let pointGraphic
      let showPoint = {
        x: vertices[i][0],
        y: vertices[i][1]
      }
      pointGraphic = this._createPoint(showPoint)
      this.map.mapView.graphics.add(pointGraphic)
    }
  }

  /**
   * 绘制面
   * @param {*} event 回调参数
   */
  _drawPolygon (event) {
    this._drawStartClick()
    let vertices = event.vertices
    this.map.mapView.graphics.removeAll()
    let startPoint = this._createPoint({ x: vertices[0][0], y: vertices[0][1] })
    if (vertices.length <= 2) {
      let startLine = this._createLine(vertices)
      this.map.mapView.graphics.addMany([startLine, startPoint])
    } else {
      this.map.mapView.graphics.removeAll()
      this.map.mapView.graphics.add(startPoint)
      let polygon = this._createPolygon(vertices)
      this.map.mapView.graphics.add(polygon)
      for (let i = 1; i < vertices.length; i++) {
        let pointGraphic
        let showPoint = {
          x: vertices[i][0],
          y: vertices[i][1]
        }
        pointGraphic = this._createPoint(showPoint)
        this.map.mapView.graphics.add(pointGraphic)
      }
    }
  }

  /**
   * 绘制圆、矩形
   * @param {*} event 回调参数
   */
  _drawSegment (event) {
    this._drawStartClick()
    let segment
    let vertices = event.vertices
    let type = this.currentDrawType
    this.map.mapView.graphics.removeAll()
    let startPoint = this._createPoint({ x: vertices[0][0], y: vertices[0][1] })
    this.map.mapView.graphics.add(startPoint)
    if (vertices.length >= 2) {
      if (type === 'circle') {
        segment = this._createCircle(vertices)
      } else if (type === 'rectangle') {
        segment = this._createRectangle(vertices)
      }
      let pointer = this._createPoint({ x: vertices[1][0], y: vertices[1][1] })
      this.map.mapView.graphics.addMany([segment, pointer])
    }
  }

  /**
   * 创建点
   * @param {*} point 点坐标
   */
  _createPoint (point) {
    return createPointGraphic(this.map, point, styleConfig.DRAW_POINT)
  }

  /**
   * 创建线
   * @param {*} vertices 坐标点集
   */
  _createLine (vertices) {
    return createLineGraphic(this.map, vertices, styleConfig.DRAW_LINE)
  }

  /**
   * 创建面
   * @param {*} vertices 坐标点集
   */
  _createPolygon (vertices) {
    let rings = [vertices]
    return createPolygonGraphic(this.map, rings, styleConfig.DRAW_POLYGON)
  }

  /**
   * 创建圆形
   * @param {*} vertices 坐标点集
   */
  _createCircle (vertices) {
    let center = new this.map.Point({
      x: vertices[0][0],
      y: vertices[0][1],
      spatialReference: this.map.sr
    })
    let radius = center.distance(new this.map.Point({
      x: vertices[1][0],
      y: vertices[1][1],
      spatialReference: this.map.sr
    }))
    return createCircleGraphic(this.map, center, radius, styleConfig.DRAW_POLYGON)
  }

  /**
   * 创建矩形
   * @param {*} vertices 坐标点集
   */
  _createRectangle (vertices) {
    return createRetangleGraphic(this.map, vertices, styleConfig.DRAW_POLYGON)
  }

  /**
   * 点击鼠标右键绘制点时撤销绘制操作
   * @param {*} event 事件
   */
  _onClickReset (event) {
    if (event.native.button === 2) {
      console.log('右击')
      this.draw.reset()
    }
  }

  /**
   * 绘制开始点击的操作
   */
  _drawStartClick () {
    this.viewManager.on({
      'click': this.drawListener
    })
  }

  /**
   * 绘制结束点击的操作
   */
  _drawEndClick () {
    this.viewManager.un({
      'click': this.drawListener
    })
  }
}
