import SgOperation from './SgOperation'

import SgViewManager from '../manage/view/SgViewManager'
import { styleConfig } from '../config/styleConfig'

export default class SgHighlightOperation extends SgOperation {
  constructor (sgMap) {
    super(sgMap)
    this.viewManager = SgViewManager.getInstance(sgMap)
  }

  /**
   * 高亮操作单例模式，通过此方法获取操作类
   * @param {*} sgMap SgMap对象实例
   */
  static getInstance (sgMap) {
    if (!this.instance) {
      this.instance = new SgHighlightOperation(sgMap)
    }
    return this.instance
  }

  /**
   * 设置几何对象高亮 传入为geometry对象，对象键值对组织形式见api geometry子类：point、polyline、polygon
   * 点=>https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html
   * 线=>https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html
   * 面=>https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html
   * @param {*} geometries 传入几何对象，可以是单个几何对象或几何对象数组
   * @param {*} zoomto 不传参或true则居中高亮要素，false则不居中高亮要素
   */
  setHighLight (geometries, zoomto) {
    if (!this.layerManager.hasLayerById(this.map.highlightLayerId)) {
      this.layerManager.addLayer(this.map.highlightLayer)
    }
    this.map.highlightLayer.removeAll()
    let graphics
    if (geometries instanceof Array) {
      graphics = []
      geometries.forEach((geometry) => {
        let graphic = this._createGraphic(geometry)
        graphics.push(graphic)
      })
      this.map.highlightLayer.addMany(graphics)
    } else {
      graphics = this._createGraphic(geometries)
      this.map.highlightLayer.add(graphics)
    }
    console.log(zoomto)
    if (zoomto !== false) {
      this.viewManager.setCenterFeature(graphics)
    }
  }

  /**
   * 清除高亮
   */
  clearHighlightLayer () {
    this.map.highlightLayer.removeAll()
  }

  /**
   * 创建要素，私有方法
   * @param {*} geometry 几何对象
   */
  _createGraphic (geometry) {
    let symbol
    if (!geometry.spatialReference) {
      geometry.spatialReference = this.map.sr
    }
    switch (geometry.type) {
      case 'point':
        symbol = styleConfig.HIGHLIGHT_POINT
        break
      case 'polyline':
        symbol = styleConfig.HIGHLIGHT_LINE
        break
      case 'polygon':
        symbol = styleConfig.HIGHLIGHT_POLYGON
        break
      default:
        return null
    }
    let graphic = this.map.graphic.clone()
    graphic.geometry = geometry
    graphic.symbol = symbol
    return graphic
  }
}
