import SgOperation from './SgOperation'
import { loadModules } from 'esri-loader'
import MAP_OPTIONS from '../config/baseConfig'
import { Subject } from 'rxjs'
import SgViewManager from '../manage/view/SgViewManager'
import SgMapOverlayer from '../overlayer/SgMapOverlayer'
import { createPointGraphic, createLineGraphic, createPolygonGraphic } from '../utils/SgMapUtils'
import { styleConfig } from '../config/styleConfig'

export default class SgMeasureOperation extends SgOperation {
  constructor (sgMap) {
    super(sgMap)
    loadModules([
      'esri/views/draw/Draw',
      'esri/geometry/geometryEngine'
    ], MAP_OPTIONS).then(([Draw, geometryEngine]) => {
      this.draw = new Draw({ view: this.map.mapView })
      this.geoEngine = geometryEngine
    })
    this.viewManager = SgViewManager.getInstance(this.map)
    this.drawLayer = this.map.drawLayer
    this.subject = new Subject()
    this.subscription = null
    this.drawListener = (event) => {
      if (event.button !== 0) {
        this.clearMeasure()
        this._drawEndClick()
      }
    }
  }

  /**
   * kelsey yau
   * 新建测量，公共方法
   * @param {*} type 测量的类型，'length'：测距，'area'：测面
   */
  createMeasure (type) {
    this.clearMeasure()
    let drawLayer = this.layerManager.getLayerById(this.map.drawLayerId)
    if (!drawLayer) { this.layerManager.addLayer(this.map.drawLayer) }
    this._initMeasure(type)
    this._drawStartClick()
  }

  /**
   * 测量结果监听
   * @param {*} eventObj 事件对象 {'measure': function() {}}
   */
  on (eventObj) {
    for (let type in eventObj) {
      this.subscription = this.subject.subscribe((value) => {
        eventObj[type](value)
      })
    }
  }

  /**
   * 销毁测量事件
   */
  destroy () {
    if (this.subscription) {
      this.subscription.unsubscribe()
      this.subscription = null
    }
  }

  /**
   * 清除测量
   */
  clearMeasure () {
    this.destroy()
    this.draw.reset()
    this.drawLayer.removeAll()
    this.map.mapView.popup.close()
  }

  /**
   * kelsey yau
   * 初始化测量，私有方法
   * @param {*} type 测量类型
   */
  _initMeasure (type) {
    this.drawLayer.graphics.removeAll()
    if (type === 'length') {
      this._createMeasureGraphic('polyline', this.draw, this.map.mapView, this._drawPolyline)
    } else if (type === 'area') {
      this._createMeasureGraphic('polygon', this.draw, this.map.mapView, this._drawPolygon)
    }
  }

  /**
   * kelsey yau
   * 创建测量绘制，私有方法
   * @param {*} type 绘制类型，'polyline'：线绘制，'polygon'：面绘制
   * @param {*} draw 绘制对象
   * @param {*} view 视图对象
   * @param {*} listener 绘制监听函数
   */
  _createMeasureGraphic (type, draw, view, listener) {
    let action = draw.create(type, { mode: 'click' })
    view.focus()
    action.on('vertex-add', listener.bind(this))
    action.on('cursor-update', listener.bind(this))
    action.on('vertex-remove', listener.bind(this))
    action.on('draw-complete', listener.bind(this))
  }

  /**
   * kelsey yau
   * 绘制线，私有方法
   * @param {*} event 事件回调参数
   */
  _drawPolyline (event) {
    let vertices = event.vertices
    this.map.drawLayer.graphics.removeAll()
    let startPoint = this._createPointGraphic({ x: vertices[0][0], y: vertices[0][1] })
    let graphic = this._createPolylineGraphic(vertices)
    this._createLabelGraphic({ x: vertices[0][0], y: vertices[0][1] }, '起点')
    this.map.drawLayer.addMany([graphic, startPoint])
    let showPoint
    let _totalDist = 0
    if (vertices.length >= 2) {
      for (let i = 1; i < vertices.length; i++) {
        let pointGraphic
        showPoint = {
          x: vertices[i][0],
          y: vertices[i][1]
        }
        let everyGraphics = this._createPolylineGraphic([vertices[i], vertices[i - 1]])
        if (this.map.sr.wkid === 4326 || this.map.sr.wkid === 4490) {
          _totalDist = _totalDist + this._getLngLatDistance(vertices[i], vertices[i - 1])
          let dist = this._formatDist(_totalDist)
          this._createLabelGraphic(showPoint, dist)
          pointGraphic = this._createPointGraphic(showPoint)
        } else {
          _totalDist = _totalDist + this.geoEngine.planarLength(everyGraphics.geometry, 'meters')
          let dist = this._formatDist(_totalDist)
          this._createLabelGraphic(showPoint, dist)
          pointGraphic = this._createPointGraphic(showPoint)
        }
        this.map.drawLayer.addMany([pointGraphic])
      }
    }
    if (event.type === 'draw-complete') {
      this.subject.next({ result: _totalDist, graphic: graphic })
      this.draw.reset()
    }
  }

  /**
   * kelsey yau
   * 绘制多边形，私有方法
   * @param {*} event object 事件回调参数
   */
  _drawPolygon (event) {
    let vertices = event.vertices
    this.drawLayer.graphics.removeAll()
    let startPoint = this._createPointGraphic({ x: vertices[0][0], y: vertices[0][1] })
    let startPolygon, area
    if (vertices.length <= 2) {
      let startPolyline = this._createPolylineGraphic(vertices)
      this.map.drawLayer.add(startPolyline)
    }
    this.map.drawLayer.add(startPoint)
    if (vertices.length > 2) {
      this.map.drawLayer.graphics.remove(startPoint)
      startPolygon = this._createPolygonGraphic(vertices)
      this.map.drawLayer.add(startPolygon)
      vertices.forEach((coor) => {
        let pointGraphic = this._createPointGraphic({ x: coor[0], y: coor[1] })
        this.map.drawLayer.add(pointGraphic)
      })
      if (this.map.sr.wkid === 4326 || this.map.sr.wkid === 4490) {
        area = this._getLngLatArea(vertices)
        this._createLabelGraphic(startPolygon.geometry.centroid, this._formatArea(area))
      } else {
        let polygon = startPolygon.geometry
        area = this.geoEngine.planarArea(polygon, 'square-meters')
        if (area < 0) {
          var simplifiedPolygon = this.geoEngine.simplify(polygon)
          if (simplifiedPolygon) {
            area = this.geoEngine.planarArea(simplifiedPolygon, 'square-meters')
          }
        }
        this._createLabelGraphic(startPolygon.geometry.centroid, this._formatArea(area))
      }
    }
    if (event.type === 'draw-complete') {
      this.subject.next({ result: area, graphic: startPolygon })
      this.draw.reset()
    }
  }

  /**
   * kelsey yau
   * 创建测量点要素，私有方法
   * @param {*} point object 点坐标对象
   */
  _createPointGraphic (point) {
    return createPointGraphic(this.map, point, styleConfig.MEASUER_POINT)
  }

  /**
   * kelsey yau
   * 创建测量线要素，私有方法
   * @param {*} vertices array 坐标集合
   */
  _createPolylineGraphic (vertices) {
    return createLineGraphic(this.map, vertices, styleConfig.MEASURE_LINE)
  }

  /**
   * kelsey yau
   * 创建测量多边形要素，私有方法
   * @param {*} vertices array 坐标集合
   */
  _createPolygonGraphic (vertices) {
    let rings = [vertices]
    return createPolygonGraphic(this.map, rings, styleConfig.MEASURE_POLYGON)
  }

  /**
   * 创建测量标注要素，私有方法
   * @param {*} point object 点坐标
   * @param {*} dist string 标注内容
   */
  _createLabelGraphic (point, dist) {
    let label = {
      type: 'point',
      x: point.x,
      y: point.y,
      spatialReference: this.map.sr
    }
    let contentLabel = document.createElement('div')
    contentLabel.innerHTML = dist
    contentLabel.style.color = '#404040'
    contentLabel.style.fontSize = '14px'
    contentLabel.style.fontWeight = 600
    contentLabel.style.paddingTop = '8px'
    SgMapOverlayer.addPopupToMap(this.map, label, { content: contentLabel, width: '100px', footer: false, header: false })
  }

  /**
   * 绘制开始点击的操作
   */
  _drawStartClick () {
    this.viewManager.on({
      'click': this.drawListener
    })
  }

  /**
   * 绘制结束点击的操作
   */
  _drawEndClick () {
    this.viewManager.un({
      'click': this.drawListener
    })
  }

  /**
   * kelsey yau
   * 格式化测距结果，私有方法
   * @param {*} dist number 测距结果
   */
  _formatDist (dist) {
    let output
    if (dist > 1000) {
      output = Math.round(dist / 1000 * 100) / 100 +
          ' ' + 'km'
    } else {
      output = Math.round(dist * 100) / 100 +
          ' ' + 'm'
    }
    return output
  }

  /**
   * kelsey yau
   * 格式化测面结果，私有方法
   * @param {*} area number 测面结果
   */
  _formatArea (area) {
    let output
    if (area > 10000) {
      output = Math.round(area / 1000000 * 100) / 100 + '<br>平方千米'
    } else {
      output = Math.round(area * 100) / 100 + '<br>平方米'
    }
    return output
  }

  /**
   * 计算两点的经纬度距离，私有方法
   * @param {*} a array 坐标a
   * @param {*} b array 坐标b
   */
  _getLngLatDistance (a, b) {
    let c = Math.PI / 180
    const d = c * a[1]
    const e = c * b[1]
    c =
      2 *
      Math.asin(
        Math.sqrt(
          Math.pow(Math.sin((d - e) / 2), 2) +
            Math.cos(d) *
              Math.cos(e) *
              Math.pow(Math.sin((c * a[0] - c * b[0]) / 2), 2)
        )
      )
    return Math.round(c * 6378137)
  }

  /**
   * 计算经纬度面积，私有方法
   * @param {*} a array 坐标集合
   */
  _getLngLatArea (a) {
    let b = 0
    let c, d, e
    let f = a.length
    const _rad = Math.PI / 180
    for (c = 0; c < f; ++c) {
      d = (c + 1) % f
      e = (c + 2) % f
      b += this._angle(a[c], a[d], a[e])
    }
    a = (f - 2) * 180
    c = b - a
    c > 420
      ? c = f * 360 - b - a
      : c > 300 && c < 420 && (c = Math.abs(360 - c))
    return c * _rad * 40680631590769
  }

  _angle (a, b, c) {
    a = this._bearing(b, a)
    b = this._bearing(b, c)
    b = a - b
    b < 0 && (b += 360)
    return b
  }

  _bearing (a, b) {
    const _deg = 180 / Math.PI
    const _rad = Math.PI / 180
    let c = _rad
    let d = a[1] * c
    let e = a[0] * c
    let f = b[1] * c
    c *= b[0]
    d = -Math.atan2(
      Math.sin(e - c) * Math.cos(f),
      Math.cos(d) * Math.sin(f) - Math.sin(d) * Math.cos(f) * Math.cos(e - c)
    )
    d < 0 && (d += Math.PI * 2)
    d *= _deg
    return d
  }
}
