import SgOperation from './SgOperation'
import { loadModules } from 'esri-loader'
import MAP_OPTIONS from '../config/baseConfig'
import SgViewManager from '../manage/view/SgViewManager'
import { Subject } from 'rxjs'
import SgMapOverlayer from '../overlayer/SgMapOverlayer'

export default class SgQueryOperation extends SgOperation {
  constructor (sgMap) {
    super(sgMap)
    this.viewManager = SgViewManager.getInstance(this.map)
    this._querySubject = new Subject()
    this._identifyClick = null
    this.sgOverlay = SgMapOverlayer.getInstance(sgMap)
    loadModules([
      'esri/tasks/IdentifyTask',
      'esri/tasks/support/IdentifyParameters',
      'esri/tasks/FindTask',
      'esri/tasks/support/FindParameters',
      'esri/tasks/QueryTask',
      'esri/tasks/support/Query'
    ], MAP_OPTIONS).then(([IdentifyTask, IdentifyParameters, FindTask, FindParameters, QueryTask, Query]) => {
      this.Query = Query
      this.QueryTask = QueryTask
      this.FindTask = FindTask
      this.FindParameters = FindParameters
      this.IdentifyTask = IdentifyTask
      this.IdentifyParameters = IdentifyParameters
    })
  }

  /**
   * 创建一个地图鼠标点击要素属性查询
   * @param {*} layerIds string || array<string> 图层id或图层id数组
   */
  createMapIdentify (layerIds) {
    this.viewManager.changeMapContainerCursor('crosshair')
    this.sgOverlay.addInfoWindow(
      { x: 0, y: 0, spatialReference: this.map.sr, type: 'point' },
      { id: 'identify', content: '左键点击查询<br/>右键点击结束' })
    this._identifyClick = (event) => {
      if (event.button === 0) {
        this._activeMapIdentify(layerIds, event.mapPoint)
      } else {
        this.unMapIdentify()
      }
    }
    this._identifyMouseMove = (event) => {
      this.sgOverlay.updateInfoWindow('identify', { x: event.x, y: event.y }, 'screen')
    }
    this.viewManager.on({
      'click': this._identifyClick,
      'pointer-move': this._identifyMouseMove
    })
  }

  /**
   * 地图鼠标点击要素属性查询结果监听
   * @param {*} callback 回调函数
   */
  onMapIdentify (callback) {
    this._querySubscroption = this._querySubject.subscribe((result) => {
      if (result.type === 'identify') {
        callback(result.value)
      }
    })
  }

  /**
   * 取消地图鼠标点击属性查询结果监听
   */
  unMapIdentify () {
    this.viewManager.un({
      'click': this._identifyClick,
      'pointer-move': this._identifyMouseMove
    })
    this.viewManager.changeMapContainerCursor('auto')
    this.sgOverlay.clearInfoWindowById('identify')
  }

  /**
   * 创建属性表查询
   * @param {*} layerIds string || array<string> 图层id或图层id数组
   * @param {*} fields string || array<string> 要查询的字段名称或多个字段数组
   * @param {*} value 属性值
   */
  creatMapAttrFind (layerIds, fields, value) {
    this._activeMapAttrFind(layerIds, fields, value)
  }

  /**
   * 属性表查询结果监听
   * @param {*} callback 回调函数
   */
  onMapAttrFind (callback) {
    this._querySubject.subscribe((result) => {
      if (result.type === 'find') {
        callback(result.value)
      }
    })
  }

  /**
   * 创建一个属性查询，与createMapAttrFind方法类似，
   * 但此方法适用于FeatureServer服务单图层，同时查询条件更完善，查询速度更快
   * @param {*} layerId string 图层的id
   * @param {*} queryOptions object 属性查询条件选项，
   * 包括
   * { queryField: 查询的字段(必填),
   *   operator: 运算符(必填), 如 >,<,>=,<=,=,like
   *   value: 值(必填),
   *   outputFields: 输出字段数组(可选，默认会输出全字段)
   * }
   * @returns {*} promise 返回一个promise
   */
  createAttributeQuery (layerId, queryOptions) {
    let queryTask = this._createQueryTask(layerId)
    let where = this._generateSqlWhere(queryOptions)
    const query = new this.Query()
    query.where = where
    queryOptions.outputFields === undefined ? query.outFields = ['*'] : query.outFields = queryOptions.outputFields
    query.returnGeometry = true
    return queryTask.execute(query)
  }

  /**
   * 创建一个空间查询，此方法适用于FeatureServer服务单图层
   * @param {*} layerId string 单个图层id
   * @param {*} queryOptions object 属性查询条件选项，与createAttributeQuery方法对应形参相同，传null则不过滤属性
   * @param {*} geometryOptions object 空间查询条件选项，包括
   * {
   *   geometry: 要进行查询的几何对象
   *   spatialRelationship: 几何空间关系，见https://developers.arcgis.com/javascript/latest/api-reference/esri-tasks-support-Query.html#spatialRelationship
   * }
   * @returns {*} promise 返回空间查询的promise
   */
  createSpatialQuery (layerId, queryOptions, geometryOptions) {
    let queryTask = this._createQueryTask(layerId)
    const query = new this.Query()
    if (queryOptions !== null) {
      let where = this._generateSqlWhere(queryOptions)
      query.where = where
      queryOptions.outputFields === undefined ? query.outFields = ['*'] : query.outFields = queryOptions.outputFields
    }
    query.returnGeometry = true
    query.geometry = geometryOptions.geometry
    query.spatialRelationship = geometryOptions.spatialRelationship
    return queryTask.execute(query)
  }

  /**
   * 创建一个统计查询，查询结果为统计值，此方法适用于FeatureServer服务单图层
   * @param {*} layerId string 单个图层id
   * @param {*} queryOptions  object 属性查询条件选项，与createAttributeQuery方法对应形参相同，传null则不过滤属性
   * @param {*} statisticOptions array<object> 统计查询条件，由统计查询条件对象构成的数组，单个统计查询对象:
   * {
   *   onStatisticField: 要进行统计的字段名称
   *   outStatisticFieldName: 输出统计结果的名称
   *   statisticType: 统计方式，有： 'count' | 'sum' | 'min' | 'max' | 'avg' | 'stddev' | 'var'
   * }
   */
  createStatisticQuery (layerId, queryOptions, statisticOptions) {
    let queryTask = this._createQueryTask(layerId)
    const query = new this.Query()
    if (queryOptions !== null) {
      let where = this._generateSqlWhere(queryOptions)
      query.where = where
      queryOptions.outputFields === undefined ? query.outFields = ['*'] : query.outFields = queryOptions.outputFields
    }
    query.returnGeometry = true
    query.outStatistics = statisticOptions
    return queryTask.execute(query)
  }

  /**
   * 激活地图鼠标点击要素属性查询工具，私有方法
   * @param {*} layerIds 图层id
   * @param {*} mapPoint 鼠标所点击的点对象
   */
  _activeMapIdentify (layerIds, mapPoint) {
    if (layerIds === null || layerIds === undefined) {
      const layerCollection = this.layerManager.getAllVisibleLayer()
      layerCollection.forEach((layer) => {
        this._excuteIdentify(layer, mapPoint, this.IdentifyTask, this.IdentifyParameters, 'visible')
      })
    } else if (layerIds instanceof Array) {
      layerIds.forEach((layerId) => {
        const layer = this.layerManager.getLayerById(layerId)
        this._excuteIdentify(layer, mapPoint, this.IdentifyTask, this.IdentifyParameters, 'visible')
      })
    } else if (typeof layerIds === 'string') {
      const layer = this.layerManager.getLayerById(layerIds)
      this._excuteIdentify(layer, mapPoint, this.IdentifyTask, this.IdentifyParameters, 'top')
    }
  }

  /**
   * 激活属性表查询
   * @param {*} layerIds 图层id
   * @param {*} fields 要查询的属性表字段
   * @param {*} value 要查询的属性值
   */
  _activeMapAttrFind (layerIds, fields, value) {
    if (layerIds === null || layerIds === undefined) {
      const layerCollection = this.layerManager.getAllVisibleLayer()
      layerCollection.forEach((layer) => {
        this._excuteFind(layer, fields, value, this.FindTask, this.FindParameters)
      })
    } else if (layerIds instanceof Array) {
      layerIds.forEach((layerId) => {
        const layer = this.layerManager.getLayerById(layerId)
        this._excuteFind(layer, fields, value, this.FindTask, this.FindParameters)
      })
    } else if (typeof layerIds === 'string') {
      const layer = this.layerManager.getLayerById(layerIds)
      this._excuteFind(layer, fields, value, this.FindTask, this.FindParameters)
    }
  }

  /**
   * 执行点击要素属性查询
   * @param {*} layer 图层对象
   * @param {*} mapPoint 点几何对象
   * @param {*} IdentifyTask identifyTask对象
   * @param {*} IdentifyParameters identifyParametes 参数对象
   * @param {*} option 查询选项：'top': 最顶部图层，'all'：所有图层，'visible'：可见图层
   */
  _excuteIdentify (layer, mapPoint, IdentifyTask, IdentifyParameters, option) {
    if (layer.url && this.layerManager.judgeArcgisQueryLayer(layer)) {
      const identifyTask = new IdentifyTask({ url: layer.url })
      let subLayers = this._getVisibieSubLayer(layer)
      const identifyParams = new IdentifyParameters({
        geometry: mapPoint,
        layerIds: subLayers,
        mapExtent: this.viewManager.getExtent(),
        width: this.viewManager.getViewContainerSize().width,
        height: this.viewManager.getViewContainerSize().height,
        layerOption: option,
        returnGeometry: true,
        tolerance: 0
      })
      identifyTask.execute(identifyParams).then((result) => {
        this._querySubject.next({
          type: 'identify',
          value: result
        })
      })
    }
  }

  /**
   * 执行属性表查询，私有方法
   * @param {*} layer 图层对象
   * @param {*} fields 属性表要查询的字段
   * @param {*} value 查询的属性值
   * @param {*} FindTask 查询对象
   * @param {*} FindParameters 查询参数对象
   */
  _excuteFind (layer, fields, value, FindTask, FindParameters) {
    if (layer.url && this.layerManager.judgeArcgisQueryLayer(layer)) {
      const findTask = new FindTask({ url: layer.url })
      let subLayer = this._getVisibieSubLayer(layer)
      if (!(fields instanceof Array)) { fields = [fields] }
      const findParams = new FindParameters({
        layerIds: subLayer,
        returnGeometry: true,
        searchFields: fields,
        searchText: value
      })
      findTask.execute(findParams).then((result) => {
        this._querySubject.next({
          type: 'find',
          value: result
        })
      })
    }
  }

  /**
   * 获取图层的可见子图层
   * @param {*} layer 图层对象
   */
  _getVisibieSubLayer (layer) {
    if (layer && layer.allSublayers) {
      let visibleSublayers = []
      layer.allSublayers.forEach((subLayer) => {
        if (subLayer.visible) {
          visibleSublayers.push(subLayer.id)
        }
      })
      return visibleSublayers
    } else {
      return null
    }
  }

  /**
   * 构建查询的条件
   */
  _generateSqlWhere (option) {
    let where = ''
    if (option.queryField) { where += option.queryField } else { return where }
    if (option.operator.toLowerCase() === 'like') {
      where += ' ' + option.operator + ' "%' + option.value + '%"'
    } else if (typeof option.value === 'string') {
      where += ' ' + option.operator + ' \'' + option.value + '\''
    } else if (typeof option.value === 'number') {
      where += ' ' + option.operator + ' ' + option.value
    }
    return where
  }

  /**
   * 构建查询对象实例
   * @param {*} layerId 单个图层Id
   */
  _createQueryTask (layerId) {
    let queryTask
    const layer = this.layerManager.getLayerById(layerId)
    if (layer.url && this.layerManager.judgeArcgisQueryLayer(layer)) {
      queryTask = new this.QueryTask({ url: layer.url })
    }
    return queryTask
  }
}
