import { loadModules } from 'esri-loader'
import MAP_OPTIONS from '../config/baseConfig'
import SgViewManager from '../manage/view/SgViewManager'

export default class SgMapOverlayer {
  constructor (sgMap) {
    this.map = sgMap
    this.infoWindows = []
    this.viewManager = SgViewManager.getInstance(sgMap)
    this.mapContainer = this.viewManager.getViewContainer()
    this._relocateInfoWindow()
  }

  /*
  * 视图管理单例模式
  * @param {*} sgMap SgMap对象实例
  */
  static getInstance (sgMap) {
    if (!this.instance) {
      this.instance = new SgMapOverlayer(sgMap)
    }
    return this.instance
  }

  /* kelsey yau
   * 给地图添加一个默认esri样式的弹窗
   * @param {*} sgMap object 传入sgMap实例对象
   * @param {*} location object || array 弹窗显示的位置，如事件回调的 event.mapPoint对象, [x, y]坐标形式
   * @param {*} popupOptions 弹窗选项
   */
  static addPopupToMap (sgMap, location, popupOptions) {
    sgMap.mapView.popup.autoOpenEnabled = false
    sgMap.mapView.popup.defaultPopupTemplateEnabled = false
    sgMap.mapView.popup.highlightEnabled = false
    sgMap.mapView.popup.open({
      location: location,
      content: popupOptions.content,
      visible: true
    })
    let popupEle = document.querySelector('.esri-popup')
    if (popupOptions.width) {
      popupEle.style.width = popupOptions.width
    }
    if (popupOptions.height) {
      popupEle.style.height = popupOptions.height
    }
    if (popupEle.children.length > 0) {
      let popupfooter = popupEle.querySelector('.esri-popup__footer')
      let popupHeader = popupEle.querySelector('.esri-popup__header')
      let popupCotent = popupEle.querySelector('.esri-popup__content')
      popupOptions.footer ? null : popupfooter.style.display = 'none'
      popupOptions.header ? null : popupHeader.style.display = 'none'
      popupCotent.style.margin = '0px !important'
    }
  }

  static closeMapPopup (sgMap) {
    sgMap.mapView.popup.close()
  }
  /**
   * kelsey yau
   * 给指定的图层添加弹窗模板
   * @param {*} layer object 图层对象
   * @param {*} titleTemp string 标题模板，弹窗的标题要显示的字段模板，具体设置见
   * https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#title
   * @param {*} contentTemp 内容显示模板，弹窗的模板是根据要显示的字段进行一个设置，具体见
   * https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#content
   */
  static addPopupTemplateToLayer (layer, titleTemp, contentTemp) {
    loadModules(['esri/PopupTemplate'], MAP_OPTIONS).then(([PopupTemplate]) => {
      const popupTemplate = new PopupTemplate({
        title: titleTemp,
        content: contentTemp
      })
      layer.popupTemplate = popupTemplate
    })
  }

  /**
   * 添加信息窗口覆盖物
   * @param {*} location 位置
   * @param {*} infoOptions 信息窗口覆盖物配置项 {id(id,必填), content(要显示的内容)}
   */
  addInfoWindow (location, infoOptions) {
    let infoEle = document.getElementById(infoOptions.id)
    if (!infoEle) { infoEle = this._createInfoWindow(infoOptions) }
    let screenCoor = this.viewManager.getScreenCoor(location)
    infoEle.style.left = screenCoor.x + 'px'
    infoEle.style.top = screenCoor.y + 'px'
    this.infoWindows.push({
      location: location,
      options: infoOptions
    })
    this.mapContainer.appendChild(infoEle)
  }

  /**
   * 更新信息窗口覆盖物位置
   * @param {*} eleId 信息窗口id
   * @param {*} location 信息窗口位置
   * @param {*} type 位置坐标类型，地图坐标：'map'，屏幕坐标：'screen'
   */
  updateInfoWindow (eleId, location, type) {
    let infoEle = document.getElementById(eleId)
    if (infoEle) {
      if (type === 'map' || type === undefined) {
        let screenCoor = this.viewManager.getScreenCoor(location)
        infoEle.style.left = screenCoor.x + 'px'
        infoEle.style.top = screenCoor.y + 'px'
      } else {
        infoEle.style.left = location.x + 'px'
        infoEle.style.top = location.y + 'px'
      }
    }
  }

  /**
   * 清除单个信息窗口覆盖物
   * @param {*} id 信息窗口覆盖物id
   */
  clearInfoWindowById (id) {
    let node = document.getElementById(id)
    // console.log(node)
    if (node.parentNode) {
      node.parentNode.removeChild(node)
    }
    let idx = this.infoWindows.findIndex((infoWindow) => {
      return infoWindow.options.id === id
    })
    this.infoWindows.splice(idx, 1)
  }

  /**
   * 清除所有信息窗口覆盖物
   */
  clearAllInfoWindows () {
    this.infoWindows.forEach((infoWindow) => {
      this.clearInfoWindowById(infoWindow.options.id)
    })
    this.infoWindows = []
  }

  /**
   * 创建信息窗口元素，私有方法
   * @param {*} infoOptions 创建配置
   */
  _createInfoWindow (infoOptions) {
    let infoWindowEle = document.createElement('div')
    infoWindowEle.style.position = 'absolute'
    infoWindowEle.style.zIndex = 100
    infoWindowEle.innerHTML = infoOptions.content
    infoWindowEle.id = infoOptions.id
    infoWindowEle.style.color = '#000'
    infoWindowEle.style.border = '1px solid #000'
    infoWindowEle.style.borderRadius = '5px'
    infoWindowEle.style.padding = '4px'
    infoWindowEle.style.backgroundColor = '#fff'
    infoWindowEle.style.fontSize = '14px'
    return infoWindowEle
  }

  _relocateInfoWindow () {
    this.map.mapView.watch('extent', () => {
      this._updateAllInfoWindows()
    })
    this.map.mapView.watch('rotation', () => {
      this._updateAllInfoWindows()
    })
  }

  _updateAllInfoWindows () {
    this.infoWindows.forEach((infoWindow) => {
      this.updateInfoWindow(infoWindow.options.id, infoWindow.location)
    })
  }
}
